﻿'All program codes are developed in the Biosimulation laboratory, Faculty of Life Sciences, Ritsumeikan University
'Reference; Publications
'A human ventricular myocyte model with refined representation of excitation-contraction coupling 
'Running title;                        'CICR in human ventricular cell model
'Authors;    'Y. Himeno†, K. Asakura†‡, C.Y. Cha†1, H. Memida†, T. Powell2, A. Amano†, and A. Noma†. 
'†Biosimulation Project, College of Life Sciences, Ritsumeikan University,
'Noji Higashi 1-1-1 Kusatsu-City, Shiga-prefecture, Japan
'‡Nippon Shinyaku, Co., Ltd., 14, Nishinosho-Monguchi-cho, Kisshoin, Minami-ku, Kyoto, Japan
'1Oxford Centre for Diabetes Endocrinology and Metabolism,
'University of Oxford, Churchill Hospital, Oxford, OX37 LJ, UK.
'2Department of Pharmacology, University of Oxford, Oxford OX1 3QT, UK

'Modifications from the published model.
'The gating mechanisms were slightly modifed according to the format used in the Kyoto model.
'The Heun method of numerical integration, which is a modified 'Euler method' is used.
'The energy metabolism is now re-implemented.


'Solution explorer lists all subclasses.
'CSVReadAndWrite.vb provides tools for Disk Write and Read
'Form.APClamp.vb ;          protocol of the Action potetial clamp experiment
'Form.General.vb;             protocol of the current clamp experiments
'Form.ICaLkinetics.vb ;    protocol to draw kinetics of LCC
'Form.RyRtransient.vb;     protocol of the state transition of the couplon model at the onset of action potential
'Form.scaleI1I2;                 protocol for simulation of EAD  by varying the inactivation rate of INaL by using the scaling factor for kI1I2, and DAD by injecting 1~1.5 mM Ca into SRup compartment
'Form.VmClampIV.vb;      protocol of Vm clamp experiment

'Graph2014.vb                   home-made tools for Graphics in 2014
'HuVECII.vb;                     All equations of HuVEC model are described in this class
'
'To select a 'Form' to run the protocol of interest, use 'My Project'  and select a Start up form (O) from the pull-down list in the solution explorer.

Imports System.IO

Public Class Form_General

    '********** prepared pens **********
    Public BlackPen1 As Pen = New Pen(Color.Black, 1)
    Public BlackPen2 As Pen = New Pen(Color.Black, 2)
    Public BluePen1 As Pen = New Pen(Color.Blue, 1)
    Public BluePen2 As Pen = New Pen(Color.Blue, 2)
    Public RedPen1 As Pen = New Pen(Color.Red, 1)
    Public RedPen2 As Pen = New Pen(Color.Red, 2)
    Public ChocolatePen1 As Pen = New Pen(Color.Chocolate, 1)
    Public ChocolatePen2 As Pen = New Pen(Color.Chocolate, 2)
    Public GreenPen1 As Pen = New Pen(Color.DarkGreen, 1)
    Public GreenPen2 As Pen = New Pen(Color.Green, 2)
    Public OlivePen1 As Pen = New Pen(Color.Olive, 1)
    Public LimePen1 As Pen = New Pen(Color.Lime, 1)
    Public LimePen2 As Pen = New Pen(Color.Lime, 2)
    Public GreenYellowPen1 As Pen = New Pen(Color.GreenYellow, 1)
    Public LightCoralPen1 As Pen = New Pen(Color.LightCoral, 1)
    Public MagentaPen1 As Pen = New Pen(Color.Magenta, 1)
    Public MagentaPen2 As Pen = New Pen(Color.Magenta, 2)
    Public VioletPen2 As Pen = New Pen(Color.Violet, 2)
    Public VioletPen1 As Pen = New Pen(Color.Violet, 1)
    Public GrayPen1 As Pen = New Pen(Color.Gray, 1)
    Public GrayPen2 As Pen = New Pen(Color.Gray, 2)
    Public DarkCyanPen1 As Pen = New Pen(Color.DarkCyan, 1)
    Public DarkMagentaPen1 As Pen = New Pen(Color.DarkMagenta, 1)
    Public OrangePen1 As Pen = New Pen(Color.Orange, 1)
    Public OrangePen2 As Pen = New Pen(Color.Orange, 2)
    Public cyanPen1 As Pen = New Pen(Color.Cyan, 1)
    Public yellowPen1 As Pen = New Pen(Color.Yellow, 1)

    Dim HV As New HuVECIII          'HV: an instance of HuVECII

    '********** Graph option **********
    Dim graphVm1 As Graph2014
    Dim graphImo2 As Graph2014
    Dim graphImi2 As Graph2014
    Dim graphConc3 As Graph2014
    Dim graphConc7 As Graph2014
    Dim graphGate4 As Graph2014
    Dim graphFb5 As Graph2014
    Dim graphTimeCourse6 As Graph2014
    Dim graphDt As Graph2014                 'ms, adaptive dt

    ''******************************   Graph2014 parameters **********************************
    Dim MinX1 As Double = 0                               'msec, Abscissa of the graph
    Dim MaxX1 As Double = 800
    Dim IntervalX1 As Double = 100

    Dim MinY1 As Double = -100                          'mV, ordinate, graph of membrane potential
    Dim MaxY1 As Double = 60
    Dim IntervalY1 As Double = 20

    Dim MinY2o As Double = -0.1                           'pA/pF, or uM, ordinate , graph of ionic Currents
    Dim MaxY2o As Double = 1.6
    Dim IntervalY2 As Double = 0.2

    Dim MinY2i As Double = -1.6                            'pA/pF, or uM, ordinate , graph of ionic Currents
    Dim MaxY2i As Double = 0.1

    Dim MinX3 As Double = 0                               'unitless, Gating variable
    Dim MaxX3 As Double = 800
    Dim IntervalX3 As Double = 100

    Dim MinY3 As Double = 0                               'mM, or uM, ordinate , graph of cytosolic Ca2+ Concentrations                                '
    Dim MaxY3 As Double = 15
    Dim IntervalY3 As Double = 1

    Dim MinY7 As Double = 0                                  'mM, or uM, ordinate , graph of SR Ca2+ Concentrations                                '
    Dim MaxY7 As Double = 1.2
    Dim IntervalY7 As Double = 0.1

    Dim MinX4 As Double = 50                              'unitless, Gating variable of RyR
    Dim MaxX4 As Double = 80

    Dim IntervalX4 As Double = 10

    Dim MinX8 As Double = 50                                  'unitless, Gating variable
    Dim MaxX8 As Double = 100
    Dim IntervalX8 As Double = 10

    Dim MinY4 As Double = 0                                  'unitless, Gating variable
    Dim MaxY4 As Double = 1
    Dim IntervalY4 As Double = 0.1

    Dim MinX5 As Double = 0
    Dim MaxX5 As Double = 800
    Dim IntervalX5 As Double = 100

    Dim MinY5 As Double = 0                                   'mN/cm2   Fb
    Dim MaxY5 As Double = 10
    Dim IntervalY5 As Double = 2

    Dim MinX6 As Double = 0                                   ' s   Time course along the repetitive stimulations
    Dim MaxX6 As Double = 200                             ' s
    Dim IntervalX6 As Double = 50

    Dim MinY6 As Double = 0
    Dim MaxY6 As Double = 10
    Dim IntervalY6 As Double = (MaxY6 - MinY6) / 10

    Dim MinY9 As Double = 0              'dt
    Dim MaxY9 As Double = 0.3
    Dim IntervalY9 As Double = 0.02

    '********** Time step **********
    Public numSweep As Double                 'counter of sweeps
    'Public dt As Double = 0.05                    ' 0.1 msec, time step 

    Public RepeatN As Double                       'Number of repeat the sweeps
    Public elapsedTime As Double               'msec, experimental time
    Public sweepTime As Double                 'msec   time for each sweep
    Dim TtoPlot As Double = 0                     'msec timing for plot the point
    Dim maxTtoPlot As Double = 1              '1 ms, interval of point plot

    '**********     counter     *********1********************************
    Dim fileNo As Integer                              'File handler
    Dim strData As String
    Dim valueArray() As String
    '***************************************** save data during simulation *******************************************************
    Dim CsvFileName As String = "CsvHuVEC.csv"
    Dim names1() As String = New String() {"Vm", "TnChCa", "CaMCa", "bufferSRCa", "Lb_jnc", "Lb_iz", "Hb_jnc", "Hb_iz", "Nai", "Ki", _
                                           "Catot_jnc", "Catot_iz", "Catot_blk", "Ca_SRup", "Catot_SRrl", "O_TM", "I2_TM", "Is_TM", "O_LSM", _
                                           "I1_LSM", "I2_LSM", "Is_LSM", "Yco_iz", "Yoc_iz", "Yoo_iz", "Yco_blk", "Yoc_blk", "Yoo_blk", "Yooo", _
                                           "Yooc", "Ycoo", "Ycoc", "Ycco", "Yoco", "Yocc", "y1_IKr", "y2_IKr", "y3_IKr", "Ov_IKs", "C2IKs_iz", _
                                           "OcIKs_iz", "C2IKs_blk", "OcIKs_blk", "a_IKto", "y1_IKto", "y2_IKto", "Pbspm", "E1NCX_iz", "I1NCX_iz", "I2NCX_iz", "E1NCX_blk", _
                                           "I1NCX_blk", "I2NCX_blk", "P1_6_NaK", "P7_NaK", "P8_13_NaK", "halfSL", "Fb", "Fp", "TSCa3", "TSCa3W", _
                                           "TSCa3S", "TSS", "TSW", "hw", "hp", "ATPt_cyt", "ADPt_cyt", "Pi_cyt", "PCr_cyt", "ATPt_mit", "Pi_mit", _
                                           "NADH_mit", "H_mit", "K_mit", "UQr_mit", "ctCrd_mit", "dpsi", "dT"}
    '****************** Variables described below for mitochondria are not used in the present version *************

    Dim IsRecord As Boolean
    Dim FileName1 As String
    Dim sw1 As StreamWriter
    Dim RecordTimeStart As Double
    Dim RecordDuration As Double
    Dim RecordInterval As Double
    Dim TtoRecord As Double

    Public Tfreeze As Double = 0
    Dim IsFreezeSlowV As Boolean = False
    Dim IsAddCaNa As Boolean = False
    Dim IsFreezeSR As Boolean = False

    Dim maxValue1 As Double = 0        'maximum rate of rise
    Dim maxValue2 As Double = 0        'peak Ca transient
    Dim maxValue3 As Double = 0        'APD90
    Dim maxValue4 As Double = 0        'peak Ca_iz
    Dim maxValue5 As Double = 0        'peak Ca_jnc
    Dim minValue1 As Double = 100

    Dim FluxL As Double                        'Sum of transition of CaRU induced by LCC activation
    Dim FluxR As Double                        'sum of transition of CaRU induced by spontaneous RyR openings

    Dim preV As Double = 0
    Dim slopeVmax As Double = 0
    Dim sumRep As Double
    Dim NsumRep As Double
    Dim FirstDraw As Boolean = True

    Public dt As Double = 0.01          ' Definition of dt in the Model, because the value of dt is modified in this method, dt is stored in CSVRW
    Public adjustStepSize As Boolean = False      'used to determine if the adaptive dt is used or not

    'Public minDT As Double = 0.02            'temporary DT for detecting minimum DT


    Private Sub ButtonStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonStart.Click
        ButtonStart.Visible = False       'hide the starting command button

        If FirstDraw = True Then                              'only when starting simulation
            DefineGraph()                                           'define graphs
            Clear()
            graphTimeCourse6.Clear()
            FirstDraw = False

            If chkDefault.Checked Then
                HV.DefaultValues()
                chkDefault.Checked = False
            Else
                CSVReadAndWrite.ReadCSV(CsvFileName, HV, names1)      'start from the last set of variables
            End If

        End If

        If chkDefault.Checked Then
            HV.DefaultValues()
        End If

        ReadForm()                                   'Stimulus strength & Sweeptime max

        'DrawAxes()
        graphTimeCourse6.DrawAxis(IntervalX6, IntervalY6, "", 8)      'plot the time course of repetitive stimulation

        numSweep = 0                                        'reset the counter of sweep
        Do While numSweep < RepeatN
            Clear()
            DrawAxes()
            ResetMeasurements()                   'initial setting for measurements, such as APD90 etc.

            sweepTime = 0                             'reset sweepTime = 0
            Do While sweepTime < HV.SInterval   'Calculate individual sweep of the simulation
                My.Application.DoEvents()      'check events

                If rbEuler.Checked Then
                    dt = 0.01
                    HV.ConvIntegration(sweepTime, dt) 'use Euler method
                End If
                If rbHeun.Checked Then HV.HeunIntegration(adjustStepSize, sweepTime, dt) 'use Heun method of integration

                ''''''''''''''''''''''''''''''''''''  measure max of Fb, Ca_blk, APD at -78 mV  etc.  ''''''''''''''''''''''''''
                MeasureMaxMin()
                ' ''''''''''''''''''''''''''''''''''''''''plot  results''''''''''''''''''''''''''''''''''''''''''''''''''''''''
                If TtoPlot >= maxTtoPlot Then                       'time to plot
                    PlotResultsA()
                    TtoPlot = 0                                                  'reset the timer
                Else
                    TtoPlot = TtoPlot + dt                                      'msec, interval of point plot
                End If

                sweepTime = sweepTime + dt                'msec, proceed with the experimental time within one sweep
                elapsedTime = elapsedTime + dt           'msec, proceed with the experimental time       
            Loop              '************************** END of one Sweep ******************************************

            minValue1 = sumRep / NsumRep               'determine the average

            If elapsedTime > MaxX6 * 1000 Then
                elapsedTime = 0 'reset the time axis
                graphTimeCourse6.Clear()
                graphTimeCourse6.DrawAxis(IntervalX6, IntervalY6, "", 8)      'plot the time course of repetitive stimulation
            End If
            plotresultsB()                                               'write final values of several variables

            WritetxtBox()

            If chkFwrite.Checked = True Then
                CSVReadAndWrite.CreateNewCSV(CsvFileName, HV, names1)   'Replace the old CSV file with a new one, and prepare the headings
                CSVReadAndWrite.WriteCSV(CsvFileName, sweepTime, HV, 1, names1)   'input = C(i), output = CsvFileName The variables of only C(1) is stored on the CSV file
            End If

            numSweep = numSweep + 1                                                             'count number of sweeps calculated
        Loop

        '**************************optional IKr block ***********************************************
        'CSVReadAndWrite.ReadCSV(CsvFileName, HV, names1)      'start from the last set of variables

        'HV.scaleIKr = 0
        'elapsedTime = 0
        'sweepTime = 0                                    'reset sweepTime = 0
        'Do While sweepTime < HV.SInterval         'Calculate individual sweep of the simulation
        '    My.Application.DoEvents()                 'check events
        '    HV.ModelCell(sweepTime, dt)             'determination of dydt(N) by calculating the model using the variable name
        '    graphVm1.PlotPoint(sweepTime, HV.Vm, RedPen)

        '    sweepTime = sweepTime + dt                                  'msec, proceed with the experimental time within one sweep
        'Loop              '************************** END of one Sweep ******************************************

        ButtonStart.Visible = True                                                      'display the start button

    End Sub

    Private Sub MeasureMaxMin()
        If sweepTime < (HV.ItestOn - 1) Then
            HV.Vrest = HV.Vm                           'Vm at the end of diastole
            HV.Carest_blk = HV.Cafree_blk        '[Ca2+] at the end of diastole
        End If
        If sweepTime < (HV.ItestOff + 10) Then HV.Vmaxpl = HV.Vm 'Vm at 10 msec after the offset of stimulus current pulse

        slopeVmax = (HV.Vm - preV) / dt     'measure the maximum rate of rise or fall
        preV = HV.Vm

        If sweepTime > HV.ItestOff + dt Then
            If slopeVmax > maxValue1 Then maxValue1 = slopeVmax 'maximum rate of rise
            If slopeVmax < minValue1 Then minValue1 = slopeVmax 'maximum rate of fall
            If HV.Vm > -80 And HV.Vm < -60 Then     'measure an average rate of fall between -30 and -70
                sumRep = sumRep + slopeVmax
                NsumRep += 1
            End If
        End If

        If HV.Cafree_blk > maxValue2 Then maxValue2 = HV.Cafree_blk 'measuring the peak of Ca transient
        If sweepTime > 100 And HV.Vm < -78 And HV.Vm > -78.5 Then maxValue3 = sweepTime - 53 'measuring APD at ~-78 mV

        If HV.Cafree_iz > maxValue4 Then maxValue4 = HV.Cafree_iz 'measuring peak  of Ca_iz
        If HV.Cafree_jnc > maxValue5 Then maxValue5 = HV.Cafree_jnc 'measuring peak search of Ca_jnc
        If HV.Ca_ndLR < HV.minCa_ndLR Then HV.minCa_ndLR = HV.Ca_ndLR 'replace the minimum Ca_ndLR

        If sweepTime > HV.VtestOn And sweepTime < HV.VtestOn + 50 Then
            FluxL = FluxL + (HV.kYoocYooo * HV.Yooc - HV.kYoooYooc * HV.Yooo) * dt
            FluxR = FluxR + (HV.kYcocYcoo * HV.Ycoc - (HV.kYcooYcoc * HV.Ycoo)) * dt
        End If
    End Sub

    Public Sub ReadForm()
        If chkAdjustdt.Checked = True Then adjustStepSize = True 'adjust dt

        If chkBS.Checked = True Then
            HV.BlinkS = True                   'The model include the blink space
            HV.ratioBS = CDbl(txtRatioBS.Text)       'ratio of gD_bs / gD_nd
        Else
            HV.BlinkS = False                   'No blink space is included in the model
        End If

        HV.Samplitude = CDbl(txtStimulus.Text)        'amplitude of the stimulus current in pA
        RepeatN = CDbl(txtRepeat.Text)                 'number of sweeps
        HV.SInterval = CDbl(txtInterval.Text)               'interval of stimulus, only necessary to measure the recovery of APD

        If chkIsometric.Checked = True Then
            HV.IsoMetric = True                                'isometric contraction
        Else
            HV.IsoMetric = False                              'isotonic contraction
        End If
        HV.isometricL = CDbl(txthalfSL.Text)          'get the halfsarcomere length from the textbox
        HV.Fext = CDbl(txtFext.Text)                        'get Fext from the textbox

        HV.sclI1I2 = CDbl(txtI1I2.Text)                    'scaling factor for kI1I2 of the late scattered mode Na channel
        HV.scaleIKr = CDbl(txtscaleIKr.Text)
        HV.scaleICaL = CDbl(txtscaleICaL.Text)

        HV.Cao = CDbl(txtCao.Text)
        HV.Nao = CDbl(txtNao.Text)
        HV.Ko = CDbl(txtKo.Text)

        HV.fNaK = CDbl(txtfNaK.Text)        'scaling the Na/K pump current 
        HV.fNaCa = CDbl(txtfNaCa.Text)     'scaling the Na/Ca exchange current 

        HV.clampMode = "Iclamp"              'always Current Clamp Mode in the present protocol

    End Sub

    Private Sub ResetMeasurements()
        maxValue1 = 0                                    'to measure the maximum value of Fb 
        maxValue2 = 0
        maxValue3 = 0
        maxValue4 = 0
        maxValue5 = 0
        HV.Fbpeak = 0
        HV.minCa_ndLR = 100
        NsumRep = 0
        sumRep = 0
        FluxL = 0
        FluxR = 0
    End Sub

    Private Sub DefineGraph()
        graphVm1 = New Graph2014(pbVm, MinX1, MaxX1, MinY1, MaxY1)
        graphImo2 = New Graph2014(pbCurrento, MinX1, MaxX1, MinY2o, MaxY2o)
        graphImi2 = New Graph2014(pbcurrenti, MinX1, MaxX1, MinY2i, MaxY2i)
        graphConc3 = New Graph2014(pbConcentration1, MinX3, MaxX3, MinY3, MaxY3)
        graphConc7 = New Graph2014(pbconcentration2, MinX1, MaxX1, MinY7, MaxY7)
        graphFb5 = New Graph2014(pbBuffer, MinX1, MaxX1, MinY5, MaxY5)
        graphTimeCourse6 = New Graph2014(pb6, MinX6, MaxX6, MinY6, MaxY6)
        'graphDt = New Graph2014(pb6, MinX1, MaxX1, MinY9, MaxY9)
    End Sub

    Private Sub DrawAxes()
        graphVm1.DrawAxis(IntervalX1, IntervalY1, "", 8)
        graphImo2.DrawAxis(IntervalX1, IntervalY2, "", 8)
        graphImi2.DrawAxis(IntervalX1, IntervalY2, "", 8)
        graphConc3.DrawAxis(IntervalX3, IntervalY3, "", 8)
        graphConc7.DrawAxis(IntervalX1, IntervalY7, "", 8)
        graphFb5.DrawAxis(IntervalX1, IntervalY5, "", 8)
        'graphDt.DrawAxis(IntervalX1, IntervalY9, "", 8)
    End Sub

    Private Sub PlotResultsA()
        graphVm1.PlotPoint(sweepTime, HV.Vm, BlackPen1)

        If chkical.Checked Then graphImi2.PlotPoint(sweepTime, HV.ICaL, BlackPen1)
        If chkinal.Checked Then graphImi2.PlotPoint(sweepTime, HV.INaL, GreenPen1)

        If chkinak.Checked Then graphImo2.PlotPoint(sweepTime, HV.INaK, RedPen1)
        If chkinaca.Checked Then graphImi2.PlotPoint(sweepTime, HV.INCX, RedPen1)

        If chkik1.Checked Then graphImo2.PlotPoint(sweepTime, HV.IK1, LimePen1)
        If chkiks.Checked Then graphImo2.PlotPoint(sweepTime, HV.IKs, ChocolatePen1)
        If chkikr.Checked Then graphImo2.PlotPoint(sweepTime, HV.IKr, BluePen1)
        If chkikto.Checked Then graphImo2.PlotPoint(sweepTime, HV.IKto, MagentaPen1)
        'graphImo2.PlotPoint(sweepTime, HV.IKpl, GrayPen1)

        If chkcablk.Checked Then graphConc3.PlotPoint(sweepTime, HV.Cafree_blk * 1000, RedPen1) 'Ion concentrations  uM
        If chkcaiz.Checked Then graphConc3.PlotPoint(sweepTime, HV.Cafree_iz * 1000, BlackPen1) 'uM
        'graphConc3.PlotPoint(sweepTime, HV.ATPt_cyt, LimePen1)

        If chksrrel.Checked Then graphConc7.PlotPoint(sweepTime, HV.Cafree_SRrl, GreenPen1)
        If chksrup.Checked Then graphConc7.PlotPoint(sweepTime, HV.Ca_SRup, ChocolatePen1)

        graphFb5.PlotPoint(sweepTime, HV.Fb, BlackPen1)

        'graphDt.PlotPoint(sweepTime, dt, BlackPen1)

    End Sub

    Public Sub plotresultsB()    'Time courses of variables
        If chkSRrlCa.checked Then graphTimeCourse6.PlotPoint(elapsedTime / 1000, HV.Cafree_SRrl * 10, GreenPen2) 'Ca_SRrl
        If chkNai.checked Then graphTimeCourse6.PlotPoint(elapsedTime / 1000, HV.Nai, BluePen2) '[Na]i
        If chkFbpk.checked Then graphTimeCourse6.PlotPoint(elapsedTime / 1000, HV.Fbpeak, RedPen2) 'peak Fb
        If chkCatpk.Checked Then graphTimeCourse6.PlotPoint(elapsedTime / 1000, maxValue2 * 10000, BlackPen2) 'peak Ca transient
        If chkAPD90.checked Then graphTimeCourse6.PlotPoint(elapsedTime / 1000, (maxValue3 - 200) / 10, VioletPen2) 'APD 90

    End Sub

    Private Sub PlotResultsC()
        If TtoPlot > maxTtoPlot Then                                                  'time to plot
            graphVm1.PlotPoint(sweepTime, HV.Vm, BlackPen1)

            graphImo2.PlotPoint(sweepTime, HV.ICaL, BlackPen1)
            graphImo2.PlotPoint(sweepTime, HV.INaL, GreenPen1)

            graphImo2.PlotPoint(sweepTime, HV.INaK, OrangePen1)
            graphImo2.PlotPoint(sweepTime, HV.INCX, RedPen1)

            graphImo2.PlotPoint(sweepTime, HV.IK1, BlackPen1)
            graphImo2.PlotPoint(sweepTime, HV.IKs, RedPen1)
            graphImo2.PlotPoint(sweepTime, HV.IKr, BluePen1)
            graphImo2.PlotPoint(sweepTime, HV.IKto, GreenPen1)
            graphImo2.PlotPoint(sweepTime, HV.IKpl, GrayPen1)

            graphConc3.PlotPoint(sweepTime, HV.Ca_ndLR * 100, RedPen1)    'Ion concentrations
            graphConc3.PlotPoint(sweepTime, HV.Ca_nd0R * 100, GreenPen1)    'Ion concentrations
            graphConc3.PlotPoint(sweepTime, HV.Ca_ndL0 * 100, BluePen1)    'Ion concentrations
            graphConc3.PlotPoint(sweepTime, HV.Ca_nd00 * 100, BlackPen1)    'Ion concentrations

            'If chkYooo.Checked Then graphGate4.PlotPoint(sweepTime, (HV.Yooo), RedPen1)
            'If chkYooc.Checked Then graphGate4.PlotPoint(sweepTime, (HV.Yooc), GreenPen1)

            'If chkYoco.Checked Then graphGate4.PlotPoint(sweepTime, (HV.Yoco), BluePen1)
            'If chkyocc.Checked Then graphGate4.PlotPoint(sweepTime, (HV.Yocc), GrayPen1)

            'If chkYcoo.Checked Then graphGate4.PlotPoint(sweepTime, (HV.Ycoo), BluePen1)
            'If chkycoc.Checked Then graphGate4.PlotPoint(sweepTime, (HV.Ycoc), BluePen1)

            'If chkYcco.Checked Then graphGate4.PlotPoint(sweepTime, (HV.Ycco), OrangePen1)
            'If chkyccc.Checked Then graphGate4.PlotPoint(sweepTime, (HV.Yccc), OrangePen1)


            graphFb5.PlotPoint(sweepTime, HV.Fb, RedPen1)
            TtoPlot = 0                                                  'reset the timer
        Else
            TtoPlot = TtoPlot + dt                                      'msec, interval of point plot
        End If
    End Sub

    Private Sub WritetxtBox()
        'txtCm.Text = Format(Cm, "###.## pF")
        txtdVmdt.Text = Format(maxValue1, "###.# V/s")    'maximum rate of rise of AP
        txtRep.Text = Format(minValue1, "##.# V/s")    'maximum rate of rise of AP
        txtNai.Text = Format(HV.Nai, "##.## mM")
        txtCapeak.Text = Format(maxValue2 * 1000, "##.## uM")
        txtCajnc.Text = Format(maxValue5 * 1000, "###.## uM")
        txtCaiz.Text = Format(maxValue4 * 1000, "###.## uM")
        'txtFbmax.Text = Format(HV.Fbpeak, "##.### mN/mm-2")
        txtAPD.Text = Format(maxValue3, "###.## ms")
        txtCa_ndLR.Text = Format(HV.minCa_ndLR * 1000, "###.## uM")
        txtCaSR.Text = Format(HV.Cafree_SRrl, "##.## mM")
        txtVrest.Text = Format(HV.Vrest, "####.### mV")
        'txtVmaxpl.Text = Format(HV.Vmaxpl, "####.### mV")
        txtCatotal.Text = Format(HV.Catot_cyt * 1000, "##0.####### ")


        txtATPcyt.Text = Format(HV.ATPt_cyt, " #.### ")
        txtATPmit.Text = Format(HV.ATPt_mit, " #.### ")
        'TextBox5.Text = Format(HV.ADPt_cyt, "ADPi = ##.##")
        'TextBox3.Text = Format(HV.Pifree_cyt, "Pi = ##.##")

        HV.Sig_INa = HV.Sig_INa + HV.INaT_Na_cyt + HV.INaL_Na_cyt
        HV.Sig_INCXNa = HV.Sig_INCXNa + HV.INCXNa_iz + HV.INCXNa_blk
        HV.Sig_allNa = HV.Sig_allNa + HV.ItotNa_cyt

        'TextBox1.Text = Format(HV.Sig_allNa, "#.##########")
        'TextBox2.Text = Format(HV.Sig_INa / HV.Sig_allNa, "##.####")
        'TextBox3.Text = Format(HV.Sig_INCXNa / HV.Sig_allNa, "##.####")
        'txtFluxL.Text = Format(FluxL, "#.######")
        'txtFluxR.Text = Format(FluxR, "#.######")
        'txtinputCm.Text = Format(HV.Cm, "###.##pF")
        'txtVcell.Text = Format(HV.Vol_cell / 1000, "###.##pL")



    End Sub

    Private Sub RecordingCondition()
        FileName1 = Path.Combine(Application.StartupPath, "xresult1.csv")
        sw1 = New StreamWriter(FileName1, False)

        If RecordTimeStart + RecordDuration > MaxX1 Then
            RecordDuration = MaxX1 - RecordTimeStart

        End If

        TtoRecord = -RecordTimeStart                 'time interval for saving the data points on the CVS file

        If RecordInterval < dt Then
            RecordInterval = dt
        End If
    End Sub

    Private Sub cbReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbReset.Click
        numSweep = RepeatN + 1
    End Sub

    Private Sub cbErase_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbErase.Click
        Clear()
        DrawAxes()
    End Sub

    Private Sub Clear()
        graphVm1.Clear()
        graphImo2.Clear()
        graphImi2.Clear()
        graphConc3.Clear()
        graphConc7.Clear()
        graphFb5.Clear()
        'graphDt.Clear()

    End Sub

    Private Sub ButtonEnd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonEnd.Click
        End
    End Sub

    Private Sub General_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

    End Sub

    Private Sub cberace2_Click(sender As Object, e As EventArgs) Handles cberace2.Click
        graphTimeCourse6.Clear()
        graphTimeCourse6.DrawAxis(IntervalX6, IntervalY6, "", 8)      'plot the time course of repetitive stimulation
        elapsedTime = 0
    End Sub

End Class
